import os
import re
import shutil
import sys

import numpy as np
import pandas as pd

import wgdi.base as base
import wgdi.collinearity as improvedcollinearity


class mycollinearity():
    def __init__(self, options):
        self.repeat_number = 20
        self.multiple = 1
        self.score = 100
        self.evalue = 1e-5
        self.blast_reverse = 'False'
        self.over_windows = 5
        self.options = options
        for k, v in options:
            setattr(self, str(k), v)
            print(str(k), ' = ', v)
        self.position = 'order'
        if hasattr(self, 'grading'):
            self.grading = [int(k) for k in self.grading.split(',')]
        else:
            self.grading = [50, 40, 25]
        if os.path.exists(self.dir):
            shutil.rmtree(self.dir)
        os.makedirs(self.dir)
        self.number = 1

    def deal_blast(self, blast, gff1, gff2, rednum, repeat_number):
        blast['grading'] = 0
        bluenum = 4+rednum
        index = [group.sort_values(by=[11], ascending=[False])[:repeat_number].index.tolist()
                 for name, group in blast.groupby([0])]
        reddata = np.array([k[:rednum] for k in index], dtype=object)
        bluedata = np.array([k[rednum:bluenum] for k in index], dtype=object)
        graydata = np.array([k[bluenum:repeat_number]
                             for k in index], dtype=object)
        if len(reddata):
            redindex = np.concatenate(reddata)
        else:
            redindex = []
        if len(bluedata):
            blueindex = np.concatenate(bluedata)
        else:
            blueindex = []
        if len(graydata):
            grayindex = np.concatenate(graydata)
        else:
            grayindex = []
        blast.loc[redindex, 'grading'] = self.grading[0]
        blast.loc[blueindex, 'grading'] = self.grading[1]
        blast.loc[grayindex, 'grading'] = self.grading[2]
        return blast[blast['grading'] > 0]

    def write_block(self, fout, data, chr1, chr2, gff1, gff2):
        gff1, gff2 = gff1[gff1['chr'] == chr1], gff2[gff2['chr'] == chr2]
        fout = open(fout, 'w')
        blocks, evalues, socres = data
        for i in range(len(blocks)):
            if len(blocks[i]) < self.over_windows:
                continue
            if blocks[i][1][0]-blocks[i][0][0] > 0:
                mark = 'plus'
            else:
                mark = 'minus'
            fout.write('# Alignment '+str(self.number)+': score='+str(socres[i])+' e_value=' + str(
                evalues[i])+' N=' + str(len(blocks[i])) + ' '+str(chr1)+'&'+str(chr2) + ' ' + mark+'\n')
            self.number += 1
            for k in blocks[i]:
                name1 = gff1[gff1['order'] == k[0]].index[0]
                name2 = gff2[gff2['order'] == k[1]].index[0]
                if gff1.loc[name1, 'stand'] == gff2.loc[name2, 'stand']:
                    order = '1'
                else:
                    order = '-1'
                s = ' '.join([name1, str(k[0]), name2, str(k[1]), order])
                fout.write(s+'\n')

    def run(self):
        lens1 = base.newlens(self.lens1, 'order')
        lens2 = base.newlens(self.lens2, 'order')
        gff1 = base.newgff(self.gff1)
        gff2 = base.newgff(self.gff2)
        gff1 = gff1[gff1['chr'].isin(lens1.index)]
        gff2 = gff2[gff2['chr'].isin(lens2.index)]
        blast = base.newblast(self.blast, int(self.score), float(
            self.evalue), gff1, gff2, self.blast_reverse)
        blast = self.deal_blast(blast, gff1, gff2, int(
            self.multiple), int(self.repeat_number))
        blast['loc1'] = blast[0].map(gff1.loc[:, self.position])
        blast['loc2'] = blast[1].map(gff2.loc[:, self.position])
        blast['chr1'] = blast[0].map(gff1.loc[:, 'chr'])
        blast['chr2'] = blast[1].map(gff2.loc[:, 'chr'])
        total = []
        for (chr1, chr2), group in blast.groupby(['chr1', 'chr2']):
            df = pd.DataFrame(np.zeros((lens1[chr1], lens2[chr2])))
            for index, row in group.iterrows():
                df.loc[row['loc1'], row['loc2']] = row['grading']
            df = df.loc[:, df.sum(axis=0) != 0]
            df = df.loc[df.sum(axis=1) != 0, :]
            collinearity = improvedcollinearity.collinearity(self.options, df)
            data = collinearity.run()
            fp = self.dir+'/'+str(chr1)+'.vs.'+str(chr2)+'.blk'
            gf1, gf2 = gff1[gff1['chr'] == chr1], gff2[gff2['chr'] == chr1]
            self.write_block(fp, data, chr1, chr2, gff1, gff2)
        args = ['cat', self.dir+'/*.blk', '>', self.savefile]
        command = ' '.join([str(k) for k in args])
        os.system(command)
        shutil.rmtree(self.dir)
        sys.exit(0)



import re
import sys

import numpy as np
import pandas as pd
import wgdi.base as base


class block_correspondence():
    def __init__(self, options):
        self.block_len = 0
        self.tandem = True
        self.pvalue = 0.05
        self.position = 'order'
        self.block_length = 5
        self.tandem_length = 200
        for k, v in options:
            setattr(self, str(k), v)
            print(k, ' = ', v)
        self.homo = [float(k) for k in self.homo.split(',')]

    def run(self):
        lens1 = base.newlens(self.lens1, self.position)
        lens2 = base.newlens(self.lens2, self.position)
        bkinfo = pd.read_csv(self.blockinfo)
        bkinfo['chr1'] = bkinfo['chr1'].astype(str)
        bkinfo['chr2'] = bkinfo['chr2'].astype(str)
        bkinfo = bkinfo[(bkinfo['length'] > int(self.block_length)) & (bkinfo['chr1'].isin(
            lens1.index)) & (bkinfo['chr2'].isin(lens2.index)) & (bkinfo['pvalue'] < float(self.pvalue))]
        cor = [[k, i, 0, lens1[i], j, 0, lens2[j], float(self.homo[0]), float(self.homo[1])] for k in range(
            1, int(self.multiple)+1) for i in lens1.index for j in lens2.index]
        cor = pd.DataFrame(
            cor, columns=['sub', 'chr1', 'start1', 'end1', 'chr2', 'start2', 'end2', 'homo1', 'homo2'])
        cor['chr1'] = cor['chr1'].astype(str)
        cor['chr2'] = cor['chr2'].astype(str)
        if self.tandem == True or self.tandem == 'true' or self.tandem == 1:
            bkinfo = self.remove_tandem(bkinfo)
        arr = self.collinearity_region(cor, bkinfo)
        bkinfo.loc[bkinfo.index.isin(arr), :].to_csv(
            self.savefile, index=False)

    def remove_tandem(self, bkinfo):
        group = bkinfo[bkinfo['chr1'] == bkinfo['chr2']].copy()
        group.loc[:, 'start'] = group.loc[:, 'start1']-group.loc[:, 'start2']
        group.loc[:, 'end'] = group.loc[:, 'end1']-group.loc[:, 'end2']
        index = group[(group['start'].abs() <= int(self.tandem_length)) | (
            group['end'].abs() <= int(self.tandem_length))].index
        bkinfo = bkinfo.drop(index)
        return bkinfo

    def collinearity_region(self, cor, bkinfo):
        arr = []
        for chr1, group in bkinfo.groupby(['chr1']):
            group = group.sort_values(by=['start1'], ascending=[True])
            for index, row in group.iterrows():
                newcor = cor[(cor['chr1'] == row['chr1']) &
                             (cor['chr2'] == row['chr2'])]
                for sub, cor_row in newcor.groupby(['sub']):
                    if row['homo'+self.multiple] <= float(cor_row['homo1']) or row['homo'+self.multiple] >= float(cor_row['homo2']):
                        continue
                    arr.append(index)
        return arr



import re
import sys

import matplotlib as mpl
import matplotlib.patches as mpatches
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import wgdi.base as base


class circos():
    def __init__(self, options):
        self.figsize = '10,10'
        self.position = 'order'
        self.label_size = 9
        self.label_radius = 0.015
        for k, v in options:
            setattr(self, str(k), v)
            print(k, ' = ', v)
        self.figsize = [float(k) for k in self.figsize.split(',')]
        self.ring_width = float(self.ring_width)

    def plot_circle(self, loc_chr, radius, color='black', lw=1, alpha=1, linestyle='-'):
        for k in loc_chr:
            start, end = loc_chr[k]
            t = np.arange(start, end, 0.005)
            x, y = (radius) * np.cos(t), (radius) * np.sin(t)
            plt.plot(x, y, linestyle=linestyle,
                     color=color, lw=lw, alpha=alpha)

    def plot_labels(self, root, labels, loc_chr, radius, horizontalalignment="center", verticalalignment="center", fontsize=6,
                    color='black'):
        for k in loc_chr:
            loc = sum(loc_chr[k]) * 0.5
            x, y = radius * np.cos(loc), radius * np.sin(loc)
            self.Wedge(root, (x, y), self.label_radius, 0,
                       360, self.label_radius, 'white', 1)
            if 1 * np.pi < loc < 2 * np.pi:
                loc += np.pi
            plt.text(x, y, labels[k], horizontalalignment=horizontalalignment, verticalalignment=verticalalignment,
                     fontsize=fontsize, color=color, rotation=0)

    def Wedge(self, ax, loc, radius, start, end, width, color, alpha):
        p = mpatches.Wedge(loc, radius, start, end, width=width,
                           edgecolor=None, facecolor=color, alpha=alpha)
        ax.add_patch(p)

    def plot_bar(self, df, radius, length, lw, color, alpha):
        for k in df[df.columns[0]].drop_duplicates().values:
            if k in ['', np.nan]:
                continue
            df_chr = df.groupby(df.columns[0]).get_group(k)
            x1, y1 = radius * \
                np.cos(df_chr['rad']), radius * np.sin(df_chr['rad'])
            x2, y2 = (radius + length) * \
                np.cos(df_chr['rad']), (radius + length) * \
                np.sin(df_chr['rad'])
            x = np.array(
                [x1.values, x2.values, [np.nan] * x1.size]).flatten('F')
            y = np.array(
                [y1.values, y2.values, [np.nan] * x1.size]).flatten('F')
            plt.plot(x, y, linestyle='-',
                     color=color[str(k)], lw=lw, alpha=alpha)

    def chr_loction(self, lens, angle_gap, angle):
        start, end, loc_chr = 0, 0.2*angle_gap, {}
        for k in lens.index:
            end += angle_gap + angle * (float(lens[k]))
            start = end - angle * (float(lens[k]))
            loc_chr[k] = [float(start), float(end)]
        return loc_chr

    def deal_alignment(self, alignment, gff, lens, loc_chr, angle):
        alignment.replace('\s+', '', inplace=True)
        alignment.replace('.', '', inplace=True)
        newalignment = alignment.copy()
        for i in range(len(alignment.columns)):
            alignment[i] = alignment[i].astype(str)
            newalignment[i] = alignment[i].map(gff['chr'].to_dict())
        newalignment['loc'] = alignment[0].map(gff[self.position].to_dict())
        newalignment[0] = newalignment[0].astype('str')
        newalignment['loc'] = newalignment['loc'].astype('float')
        newalignment = newalignment[newalignment[0].isin(lens.index) == True]
        newalignment['rad'] = np.nan
        for name, group in newalignment.groupby([0]):
            if str(name) not in loc_chr:
                continue
            newalignment.loc[group.index, 'rad'] = loc_chr[str(
                name)][0]+angle * group['loc']
        return newalignment

    def deal_ancestor(self, alignment, gff, lens, loc_chr, angle, al):
        alignment.replace('\s+', '', inplace=True)
        alignment.replace('.', np.nan, inplace=True)
        newalignment = pd.merge(alignment, gff, left_on=0, right_on=gff.index)
        newalignment['rad'] = np.nan
        for name, group in newalignment.groupby(['chr']):
            if str(name) not in loc_chr:
                continue
            newalignment.loc[group.index, 'rad'] = loc_chr[str(
                name)][0]+angle * group[self.position]
        newalignment.index = newalignment[0]
        newalignment[0] = newalignment[0].map(newalignment['rad'].to_dict())
        data = []
        for index_al, row_al in al.iterrows():
            for k in alignment.columns[1:]:
                alignment[k] = alignment[k].astype(str)
                group = newalignment[(newalignment['chr'] == row_al['chr']) & (
                    newalignment['order'] >= row_al['start']) & (newalignment['order'] <= row_al['end'])]
                group[k] = group[k].map(newalignment['rad'].to_dict())
                group[k].dropna(inplace=True)
                group.index = group.index.map(newalignment['rad'].to_dict())
                group['color'] = row_al['color']
                group = group[group[k].notnull()]
                data += group[[0,k,'color']].values.tolist()
        df = pd.DataFrame(data, columns=['loc1', 'loc2', 'color'])
        return df

    def plot_collinearity(self, data, radius, lw=0.02, alpha=1):
        for name, group in data.groupby('color'):
            x,y = np.array([]), np.array([])
            for index, row in group.iterrows():
                ex1x, ex1y = radius * \
                    np.cos(row['loc1']), radius*np.sin(row['loc1'])
                ex2x, ex2y = radius * \
                    np.cos(row['loc2']), radius*np.sin(row['loc2'])
                ex3x, ex3y = radius * (1-abs(row['loc1']-row['loc2'])/np.pi) * np.cos((row['loc1']+row['loc2'])*0.5), radius * (
                    1-abs(row['loc1']-row['loc2'])/np.pi) * np.sin((row['loc1']+row['loc2'])*0.5)
                x1 = [ex1x, 0.5*ex3x, ex2x]
                y1 = [ex1y, 0.5*ex3y, ex2y]
                step = .002
                t = np.arange(0, 1+step, step)
                xt = base.Bezier3(x1, t)
                yt = base.Bezier3(y1, t)
                x = np.hstack((x,xt,np.nan))
                y = np.hstack((y,yt,np.nan))
            plt.plot(x, y, color=name, lw=lw,alpha=alpha)

    def run(self):
        fig = plt.figure(figsize=(tuple(self.figsize)))
        root = plt.axes([0, 0, 1, 1])
        mpl.rcParams['agg.path.chunksize'] = 100000000
        lens = base.newlens(self.lens, self.position)
        radius, angle_gap = float(self.radius), float(self.angle_gap)
        angle = (2 * np.pi - (int(len(lens))+1.5) * angle_gap) / (int(lens.sum()))
        loc_chr = self.chr_loction(lens, angle_gap, angle)
        list_colors = [str(k).strip() for k in re.split(',|:', self.colors)]
        chr_color = dict(zip(list_colors[::2], list_colors[1::2]))
        for k in loc_chr:
            start, end = loc_chr[k]
            self.Wedge(root, (0.0, 0.0), radius+self.ring_width, start * 180 /
                       np.pi, end * 180 / np.pi, self.ring_width*0.3, chr_color[k], 0.9)
        gff = base.newgff(self.gff)
        if hasattr(self, 'ancestor'):
            ancestor = pd.read_csv(self.ancestor, sep='\t', header=None)
            al = pd.read_csv(self.ancestor_location, sep='\t', header=None)
            al.rename(columns={0: 'chr', 1: 'start',
                               2: 'end', 3: 'color'}, inplace=True)
            al['chr'] = al['chr'].astype(str)
            data = self.deal_ancestor(ancestor, gff, lens, loc_chr, angle, al)
            self.plot_collinearity(data, radius, lw=0.1, alpha=0.8)

        if hasattr(self,'alignment'):
            alignment = pd.read_csv(self.alignment, sep='\t', header=None)
            newalignment = self.deal_alignment(
                alignment, gff, lens, loc_chr, angle)
            names = [str(k) for k in self.column_names.split(',')]
            n=0
            align = dict(family='Arial', verticalalignment="center", horizontalalignment="center")
            for k, v in enumerate(newalignment.columns[1:-2]):
                r = radius + self.ring_width*(k+1)
                print(r)
                self.plot_circle(loc_chr, r, lw=0.5, alpha=1, color='grey')
                self.plot_bar(newalignment[[v, 'rad']], r + self.ring_width *
                            0.15, self.ring_width*0.7, 0.15, chr_color, 1)
                if n%2==0:
                    loc=0.05
                    x, y = (r+self.ring_width*0.5) * np.cos(loc), (r+self.ring_width*0.5) * np.sin(loc)
                    plt.text(x,y,names[n],rotation=loc* 180 / np.pi,fontsize=10,**align)
                else:
                    loc=-0.08
                    x, y = (r+self.ring_width*0.5) * np.cos(loc), (r+self.ring_width*0.5) * np.sin(loc)
                    plt.text(x,y,names[n],fontsize=10,rotation=loc* 180 / np.pi,**align)
                n+=1
        labels = self.chr_label + lens.index
        labels = dict(zip(lens.index, labels))
        self.plot_labels(root,labels, loc_chr, radius - self.ring_width*0.3, fontsize=self.label_size)
        root.set_xlim(-1, 1)
        root.set_ylim(-1.05, 0.95)
        root.set_axis_off()
        plt.savefig(self.savefig, dpi=500)
        plt.show()
        sys.exit(0)



import re
import sys

import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
from scipy import stats


class ksfigure():
    def __init__(self, options):
        self.figsize = 10, 6.18
        self.legendfontsize = 9
        self.labelfontsize = 9
        self.area = 0, 3
        self.mode = 'median'
        for k, v in options:
            setattr(self, str(k), v)
            print(str(k), ' = ', v)
        if self.xlabel == 'none' or self.xlabel == '':
            self.xlabel = r'Synonymous nucleotide subsititution (${K_{s}}$)'
        if self.ylabel == 'none' or self.ylabel == '':
            self.ylabel = 'No. of syntenic blocks kernel density'
        if self.title == 'none' or self.title == '':
            self.title = 'ks'
        self.figsize = [float(k) for k in self.figsize.split(',')]
        self.area = [float(k) for k in self.area.split(',')]

    def Gaussian_distribution(self, t, k):
        y = np.zeros(len(t))
        for i in range(0, int((len(k) - 1) / 3)+1):
            if np.isnan(k[3 * i + 2]):
                continue
            y1 = stats.norm.pdf(
                t, float(k[3 * i + 1]), float(k[3 * i + 2])) * float(k[3 * i + 0])
            y = y+y1
        return y

    def run(self):
        plt.rcParams['ytick.major.pad'] = 0
        fig, ax = plt.subplots(figsize=self.figsize)
        ksfit = pd.read_csv(self.ksfit, sep='\t', index_col=0)
        t = np.arange(self.area[0], self.area[1], 0.005)
        col = [k for k in ksfit.columns if re.match('Unnamed:', k)]
        for index, row in ksfit.iterrows():
            ax.plot(t, self.Gaussian_distribution(
                t, row[col].values), linestyle=row['linestyle'], color=row['color'], label=index, linewidth=row['linewidth'])

        align = dict(family='Arial', verticalalignment="center",
                     horizontalalignment="center")
        ax.set_xlabel(self.xlabel, fontsize=self.labelfontsize,
                      labelpad=20, **align)
        ax.set_ylabel(self.ylabel, fontsize=self.labelfontsize,
                      labelpad=20, **align)
        ax.set_title(self.title, weight='bold',
                     fontsize=self.labelfontsize, **align)
        plt.tick_params(labelsize=10)
        plt.legend(loc='upper right', fontsize=self.legendfontsize,
                   prop={'family': 'Arial', 'style': 'italic'}, )
        plt.gca().spines['top'].set_visible(False)
        plt.gca().spines['right'].set_visible(False)
        plt.savefig(self.savefig, dpi=300)
        plt.show()
        sys.exit(0)




import re
import sys

import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
from scipy.stats.kde import gaussian_kde

import wgdi.base as base


class kspeaks():
    def __init__(self, options):
        self.tandem_length = 200
        self.figsize = 10, 6.18
        self.fontsize = 9
        self.block_length = 3
        self.area = 0, 3
        for k, v in options:
            setattr(self, str(k), v)
            print(str(k), ' = ', v)
        self.homo = [float(k) for k in self.homo.split(',')]
        self.ks_area = [float(k) for k in self.ks_area.split(',')]
        self.figsize = [float(k) for k in self.figsize.split(',')]
        self.area = [float(k) for k in self.area.split(',')]

    def remove_tandem(self, bkinfo):
        group = bkinfo[bkinfo['chr1'] == bkinfo['chr2']].copy()
        group.loc[:, 'start'] = group.loc[:, 'start1']-group.loc[:, 'start2']
        group.loc[:, 'end'] = group.loc[:, 'end1']-group.loc[:, 'end2']
        index = group[(group['start'].abs() <= int(self.tandem_length)) | (
            group['end'].abs() <= int(self.tandem_length))].index
        bkinfo = bkinfo.drop(index)
        return bkinfo

    def ks_kde(self, df):
        ks = df['ks'].str.split('_')
        arr = []
        ks_ave = []
        for v in ks.values:
            v = [float(k) for k in v if float(k) >= 0]
            if len(v) == 0:
                continue
            arr.extend(v)
            ks_ave.append(sum([float(k) for k in v])/len(v))
        kdemedian = gaussian_kde(df['ks_median'].values)
        kdemedian.set_bandwidth(bw_method=kdemedian.factor / 3.)
        kdeaverage = gaussian_kde(ks_ave)
        kdeaverage.set_bandwidth(bw_method=kdeaverage.factor / 3.)
        kdetotal = gaussian_kde(arr)
        kdetotal.set_bandwidth(bw_method=kdetotal.factor / 3.)
        return [kdemedian, kdeaverage, kdetotal]

    def run(self):
        plt.rcParams['ytick.major.pad'] = 0
        fig, ax = plt.subplots(figsize=self.figsize)
        bkinfo = pd.read_csv(self.blockinfo)
        bkinfo['chr1'] = bkinfo['chr1'].astype(str)
        bkinfo['chr2'] = bkinfo['chr2'].astype(str)
        bkinfo = bkinfo[(bkinfo['length'] > int(self.block_length))
                        & (bkinfo['pvalue'] < float(self.pvalue))]
        if self.tandem == True or self.tandem == 'true' or self.tandem == 1:
            bkinfo = self.remove_tandem(bkinfo)
        bkinfo = bkinfo[bkinfo['length'] > int(self.block_length)]
        bkinfo = bkinfo[bkinfo['homo' +
                               str(self.multiple)] > float(self.homo[0])]
        bkinfo = bkinfo[bkinfo['homo' +
                               str(self.multiple)] < float(self.homo[1])]
        bkinfo = bkinfo[bkinfo['ks_median'] > float(self.ks_area[0])]
        bkinfo = bkinfo[bkinfo['ks_median'] < float(self.ks_area[1])]
        kdemedian, kdeaverage, kdetotal = self.ks_kde(bkinfo)
        dist_space = np.linspace(self.area[0], self.area[1], 500)
        ax.plot(dist_space, kdemedian(dist_space), color='red',
                label='block median')
        ax.plot(dist_space, kdeaverage(dist_space),
                color='black', label='block average')
        ax.plot(dist_space, kdetotal(dist_space),
                color='blue', label='all pairs')
        ax.grid()
        align = dict(family='Arial', verticalalignment="center",
                     horizontalalignment="center")
        ax.set_xlabel(r'${K_{s}}$', fontsize=20)
        ax.set_ylabel('Frequency', fontsize=20)
        ax.tick_params(labelsize=18)
        ax.legend(fontsize=20)
        plt.subplots_adjust(left=0.09, right=0.96, top=0.93, bottom=0.12)
        plt.savefig(self.savefig, dpi=500)
        plt.show()
        bkinfo.to_csv(self.savefile, index=False)
        sys.exit(0)



import configparser
import os
import re

import numpy as np
import pandas as pd
import wgdi
from Bio import Seq, SeqIO, SeqRecord


def config():
    conf = configparser.ConfigParser()
    conf.read(os.path.join(wgdi.__path__[0], 'conf.ini'))
    return conf.items('ini')


def load_conf(file, section):
    conf = configparser.ConfigParser()
    conf.read(file)
    return conf.items(section)


def read_colinearscan(file):
    data, b, flag, num = [], [], 0, 1
    with open(file) as f:
        for line in f.readlines():
            line = line.strip()
            if re.match(r"the", line):
                num = re.search('\d+', line).group()
                b = []
                flag = 1
                continue
            if re.match(r"\>LOCALE", line):
                flag = 0
                p = re.split(':', line)
                if len(b) > 0:
                    data.append([num, b, p[1]])
                b = []
                continue
            if flag == 1:
                a = re.split(r"\s", line)
                b.append(a)
    return data

def read_mcscanx(fn):
    f1=open(fn)   
    data,b=[],[]
    flag,num=0,0
    for line in f1.readlines():
        line=line.strip()
        if re.match(r"## Alignment",line):
            flag=1
            if len(b)==0:
                arr= re.findall(r"[\d+\.]+", line)[0]
                continue
            data.append([num,b,0])
            b=[]
            num = re.findall(r"\d+", line)[0]
            continue
        if flag==0:
            continue
        a=re.split(r"\:",line)
        c=re.split(r"\s+",a[1])
        b.append([c[1],c[1],c[2],c[2]])
    data.append([num,b,0])
    return data

def read_coliearity(fn):
    f1=open(fn)   
    data,b=[],[]
    flag,num=0,0
    for line in f1.readlines():
        line=line.strip()
        if re.match(r"# Alignment",line):
            flag=1
            if len(b)==0:
                arr =re.findall('[\.\d+]+',line)
                continue
            data.append([arr[0],b,arr[2]])
            b=[]
            arr =re.findall('[\.\d+]+',line)
            continue
        if flag==0:
            continue
        b.append(re.split(r"\s", line))
    data.append([arr[0],b,arr[2]])
    return data

def read_ks(file, col):
    ks = pd.read_csv(file, sep='\t')
    ks.drop_duplicates(subset=['id1', 'id2'], keep='first', inplace=True)
    ks[col] = ks[col].astype(float)
    ks = ks[ks[col] >= 0]
    ks.index = ks['id1']+','+ks['id2']
    return ks[col]


def get_median(data):
    if len(data) == 0:
        return 0
    data.sort()
    half = len(data) // 2
    return (data[half] + data[~half]) / 2


def cds_to_pep(cds_file, pep_file, fmt='fasta'):
    records = list(SeqIO.parse(cds_file, fmt))
    for k in records:
        k.seq = k.seq.translate()
    SeqIO.write(records, pep_file, 'fasta')
    return True


def newblast(file, score, evalue, gene_loc1, gene_loc2, reverse):
    blast = pd.read_csv(file, sep="\t", header=None)
    if reverse.upper() == 'TRUE':
        blast[[0, 1]] = blast[[1, 0]]
    blast = blast[(blast[11] >= score) & (
        blast[10] < evalue) & (blast[1] != blast[0])]
    blast = blast[(blast[0].isin(gene_loc1.index)) &
                  (blast[1].isin(gene_loc2.index))]
    blast.drop_duplicates(subset=[0, 1], keep='first', inplace=True)
    blast[0] = blast[0].astype(str)
    blast[1] = blast[1].astype(str)
    return blast


def newgff(file):
    gff = pd.read_csv(file, sep="\t", header=None, index_col=1)
    gff.rename(columns={0: 'chr', 2: 'start',
                        3: 'end', 4: 'stand', 5: 'order'}, inplace=True)
    gff['chr'] = gff['chr'].astype(str)
    gff['start'] = gff['start'].astype(int)
    gff['end'] = gff['end'].astype(int)
    gff['stand'] = gff['stand'].astype(str)
    gff['order'] = gff['order'].astype(int)
    return gff


def newlens(file, position):
    lens = pd.read_csv(file, sep="\t", header=None, index_col=0)
    lens.index = lens.index.astype(str)
    if position == 'order':
        lens = lens[2]
    if position == 'end':
        lens = lens[1]
    return lens


def gene_location(gff, lens, step, position):
    loc_gene, dict_chr, n = {}, {}, 0
    gff = gff[gff['chr'].isin(lens.index)].copy()
    dict_chr = dict(zip(lens.index, np.append(
        np.array([0]), lens.cumsum()[:-1].values)))
    gff.loc[:, 'loc'] = ''
    for name, group in gff.groupby(['chr']):
        gff.loc[group.index, 'loc'] = (dict_chr[name]+group[position])*step
    return gff


def dotplot_frame(fig, ax, lens1, lens2, step1, step2, genome1_name, genome2_name, arr):
    for k in lens1.cumsum()[:-1]*step1:
        ax.axhline(y=k, alpha=0.8, color='black', lw=0.5)
    for k in lens2.cumsum()[:-1]*step2:
        ax.axvline(x=k, alpha=0.8, color='black', lw=0.5)
    align = dict(family='Arial', style='normal',
                 horizontalalignment="center", verticalalignment="center")
    align1 = dict(family='Arial', style='normal',
                  horizontalalignment="right", verticalalignment="center")
    yticks = lens1.cumsum()*step1-0.5*lens1*step1
    ax.set_yticks(yticks)
    ax.set_yticklabels(lens1.index, fontsize=12, **align1)
    xticks = lens2.cumsum()*step2-0.5*lens2*step2
    ax.set_xticks(xticks)
    # ax.set_xticks([])
    # ax.set_yticks([])
    ax.set_xticklabels(lens2.index, fontsize=12, **align)
    ax.xaxis.set_ticks_position('none')
    ax.yaxis.set_ticks_position('none')
    if arr[0] < 0:
        ax.text(-0.065, 0.5, genome1_name, weight='semibold',
                fontsize=18, rotation=90, **align)
    elif arr[0] == 1:
        ax.text(-0.065, 0.5, genome1_name, weight='semibold',
                fontsize=18, rotation=90, **align)
    else:
        ax.text(-0.05, 0.5, genome1_name, weight='semibold',
                fontsize=18, rotation=90, **align)
    if arr[1] < 0:
        ax.text(0.5, -0.065, genome2_name,
                weight='semibold', fontsize=18, **align)
    else:
        ax.text(0.5, -0.05, genome2_name,
                weight='semibold', fontsize=18, **align)


def Bezier3(plist, t):
    p0, p1, p2 = plist
    return p0*(1-t)**2+2*p1*t*(1-t)+p2*t**2


def Bezier4(plist, t):
    p0, p1, p2, p3, p4 = plist
    return p0*(1-t)**4+4*p1*t*(1-t)**3+6*p2*t**2*(1-t)**2+4*p3*(1-t)*t**3+p4*t**4



import re

import numpy as np
import pandas as pd

from wgdi import base


class collinearity:
    def __init__(self, options, matrix):
        self.gap_penality = -1
        self.over_length = 100000
        self.mg1 = 25
        self.mg2 = 25
        self.over_gap = 5
        self.path_dict = {}
        self.mat = matrix
        for k, v in options:
            setattr(self, str(k), v)
        if hasattr(self, 'mg'):
            self.mg1, self.mg2 = [int(k) for k in self.mg.split(',')]
        else:
            self.mg1, self.mg2 = [25, 25]

    def get_martix(self):
        self.mat.columns = self.mat.columns.astype(int)
        self.mat.index = self.mat.index.astype(int)
        (m, n) = self.mat.shape
        self.score1 = self.mat.copy()
        self.score2 = self.mat.copy()
        self.mat_new = self.mat.copy()
        self.matold = self.mat.copy()
        self.path1 = pd.DataFrame([['' for i in range(n)] for j in range(
            m)], index=self.mat.index, columns=self.mat.columns)
        self.path2 = pd.DataFrame([['' for i in range(n)] for j in range(
            m)], index=self.mat.index, columns=self.mat.columns)

    def run(self):
        self.get_martix()
        loc, evalues, scores = [], [], []
        while(self.over_length >= 3):
            if self.maxPath():
                loc.append(self.path)
                evalues.append(self.evalue)
                scores.append(self.score)
        return loc, evalues, scores

    def maxPath(self):
        mat_new_index, mat_new_columns = self.mat_new.index, self.mat_new.columns
        for i, row in enumerate(mat_new_index):
            for j, col in enumerate(mat_new_columns):
                if self.mat_new.loc[row, col] == 0:
                    continue
                for row_i in mat_new_index[i+1:i+1+self.mg1]:
                    if row_i - row > self.mg1:
                        break
                    for col_j in mat_new_columns[j+1:j+1+self.mg2]:
                        if col_j - col > self.mg2:
                            break
                        if self.mat_new.loc[row_i, col_j] == 0:
                            continue
                        s = self.score1.loc[row, col]+self.mat_new.loc[row_i,
                                                                       col_j]+(row_i-row-1+col_j-col-1)*self.gap_penality
                        if self.score1.loc[row_i, col_j] < s:
                            self.score1.loc[row_i, col_j] = s
                            self.path1.loc[row_i,
                                           col_j] = self.path1.loc[row, col]
                            self.path1.loc[row_i,
                                           col_j] += str(row)+':'+str(col)+'_'
        mat_new_index = mat_new_index[::-1]
        for i, row in enumerate(mat_new_index):
            for j, col in enumerate(mat_new_columns):
                if self.mat_new.loc[row, col] == 0:
                    continue
                for row_i in mat_new_index[i+1:i+1+self.mg1]:
                    if row - row_i > self.mg1 or row_i >= row:
                        break
                    for col_j in mat_new_columns[j+1:j+1+self.mg2]:
                        if col_j - col > self.mg2:
                            break
                        if self.mat_new.loc[row_i, col_j] == 0:
                            continue
                        s = self.score2.loc[row, col]+self.mat_new.loc[row_i,
                                                                       col_j]+(row-row_i-1+col_j-col-1)*self.gap_penality
                        if self.score2.loc[row_i, col_j] < s:
                            self.score2.loc[row_i, col_j] = s
                            self.path2.loc[row_i,
                                           col_j] = self.path2.loc[row, col]
                            self.path2.loc[row_i,
                                           col_j] += str(row)+':'+str(col)+'_'

        if self.score1.empty or self.score2.empty or self.score1.stack().max() == self.score2.stack().max() == 0:
            self.over_length = 0
            self.path = []
            self.evalue = np.nan
            self.score = 0
            return False
        if self.score1.stack().max() >= self.score2.stack().max():
            (x, y) = self.score1.stack().idxmax()
            self.score = self.score1.loc[x, y]
            self.path1.loc[x, y] += str(int(x))+':'+str(y)+'_'
            array = re.findall('\d+', self.path1.loc[x, y])
            self.path = [[int(array[i]), int(array[i+1])]
                         for i in range(0, len(array), 2)]
        else:
            (x, y) = self.score2.stack().idxmax()
            self.score = self.score2.loc[x, y]
            self.path2.loc[x, y] += str(int(x))+':'+str(y)+'_'
            array = re.findall('\d+', self.path2.loc[x, y])
            self.path = [[int(array[i]), int(array[i+1])]
                         for i in range(0, len(array), 2)]
        self.over_length = len(self.path)
        (x1, y1), (x2, y2) = self.path[0], self.path[-1]
        x1, x2 = sorted([x1, x2])
        y1, y2 = sorted([y1, y2])
        x_gap, y_gap = [], []
        for k in self.mat.index:
            if k <= x2+self.mg1 and k >= x1 - self.mg1:
                x_gap.append(k)
        for k in self.mat.columns:
            if k <= y2 + self.mg2 and k >= y1-self.mg2:
                y_gap.append(k)
        del_x = [k for k in self.mat.index if x1 <= k <= x2]
        del_y = [k for k in self.mat.columns if y1 <= k <= y2]
        mark = False
        if self.right_path():
            for row, col in self.path:
                self.mat.loc[row, col] = 0
        else:
            mark = True
        self.score1.loc[x_gap, y_gap] = 0
        self.score2.loc[x_gap, y_gap] = 0
        self.path1.loc[x_gap, y_gap] = ''
        self.path2.loc[x_gap, y_gap] = ''
        self.mat_new = self.mat.loc[x_gap, y_gap]
        self.mat_new = self.mat_new.loc[:, self.mat_new.sum(axis=0) != 0]
        self.mat_new = self.mat_new.loc[self.mat_new.sum(axis=1) != 0, :]
        if mark:
            return False
        self.evalue = self.evalue_estimated()
        return True

    def right_path(self):
        for k in self.path:
            if str(k[0])+':'+str(k[1]) in self.path_dict:
                return False
        for k in self.path:
            self.path_dict[str(k[0])+':'+str(k[1])] = 1
        return True

    def evalue_estimated(self):
        (x1, y1), (x2, y2) = self.path[0], self.path[-1]
        x1, x2 = sorted([x1, x2])
        y1, y2 = sorted([y1, y2])
        N = 0
        for (x, y) in self.path:
            mat = self.matold.loc[x, self.matold.columns.isin(
                range(y-int(self.mg2*0.5), y+int(self.mg2*0.5)))]
            mat[mat > 0] = 1
            N += mat.sum()
        m = len(self.path)
        L1, L2 = x2-x1+1, y2-y1+1
        a = (N-m+1)/N*(L1-m+1)*(L2-m+1)/L1/L2/m
        return round(a, 4)



import fileinput
import os
import re
import sys
from io import StringIO

import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
from Bio import AlignIO, SeqIO
from Bio.Align.Applications import MafftCommandline, MuscleCommandline

import wgdi.base as base


class trees():
    def __init__(self, options):
        base_conf = base.config()
        self.position = 'order'
        self.alignfile = ''
        self.cdsfile = ''
        for k, v in base_conf:
            setattr(self, str(k), v)
        for k, v in options:
            setattr(self, str(k), v)
            print(str(k), ' = ', v)

    def grouping(self, alignment):
        data = []
        if not os.path.exists(self.dir):
            os.makedirs(self.dir)
        cds = SeqIO.to_dict(SeqIO.parse(self.cds_file, "fasta"))
        for index, row in alignment.iterrows():
            file = str(row['chr'])+'g'+str(row[self.position])
            self.cdsfile = os.path.join(self.dir, file+'.fasta')
            self.alignfile = os.path.join(self.dir, file+'.aln')
            if os.path.isfile(self.alignfile):
                data.append(self.alignfile)
                continue
            ids = []
            for i in range(len(row[:-2])):
                if row[i] is np.nan:
                    continue
                gene_cds = cds[row[i]]
                gene_cds.id = str(int(i)+1)
                ids.append(gene_cds)

            SeqIO.write(ids, self.cdsfile, "fasta")
            self.align()
            self.buildtrees()
            data.append(self.alignfile)
        return data

    def align(self):
        if self.align_software == 'mafft':
            mafft_cline = MafftCommandline(
                cmd=self.mafft_path, input=self.cdsfile, auto=True)
            stdout, stderr = mafft_cline()
            align = AlignIO.read(StringIO(stdout), "fasta")
            AlignIO.write(align, self.alignfile, "fasta")
        if self.align_software == 'muscle':
            muscle_cline = MuscleCommandline(
                cmd=self.muscle_path, input=self.cdsfile, out=self.alignfile, seqtype="nucleo", clwstrict=True)
            stdout, stderr = muscle_cline()

    def buildtrees(self):
        args = [self.iqtree_path, '-s', self.alignfile, '-m', self.model]
        command = ' '.join(args)
        try:
            os.system(command)
        except:
            return False
        return True

    def run(self):
        alignment = pd.read_csv(self.alignment, header=None)
        alignment.replace('.', np.nan, inplace=True)
        gff = base.newgff(self.gff)
        lens = base.newlens(self.lens, self.position)
        gff = gff[gff['chr'].isin(lens.index)]
        alignment.dropna(thresh=3, inplace=True)
        alignment = pd.merge(
            alignment, gff[['chr', self.position]], left_on=0, right_on=gff.index, how='left')
        data = self.grouping(alignment)
        data = [k+'.treefile' for k in data]
        fout = open(self.trees_file, 'w')
        fout.close()
        for i in range(0, len(data), 100):
            trees = ' '.join([str(k) for k in data[i:i+100]])
            args = ['cat', trees, '>>', self.trees_file]
            command = ' '.join([str(k) for k in args])
            os.system(command)
        df = pd.read_csv(self.trees_file, header=None, sep='\t')
        df[1] = data
        df[[1, 0]].to_csv(self.trees_file, index=None, sep='\t', header=False)



import os
import re
import sys
from io import StringIO

import numpy as np
import pandas as pd
import wgdi.base as base
from Bio import AlignIO, SeqIO
from Bio.Align.Applications import MafftCommandline, MuscleCommandline
from Bio.Phylo.PAML import yn00


class ks():
    def __init__(self, options):
        base_conf = base.config()
        self.pair_pep_file = 'pair.pep'
        self.pair_cds_file = 'pair.cds'
        self.prot_align_file = 'prot.aln'
        self.mrtrans = 'pair.mrtrans'
        self.pair_yn = 'pair.yn'
        self.cds_file = 'cds'
        self.pep_file = 'pep'
        for k, v in base_conf:
            setattr(self, str(k), v)
        for k, v in options:
            setattr(self, str(k), v)
            print(str(k), ' = ', v)

    def auto_file(self):
        pairs = []
        p = pd.read_csv(self.pairs_file, sep='\n', header=None, nrows=30)
        p = '\n'.join(p[0])
        if 'path length' in p or 'MAXIMUM GAP' in p:
            collinearity = base.read_colinearscan(self.pairs_file)
            pairs = [[v[0], v[2]] for k in collinearity for v in k[1]]
        elif '# Alignment' in p:
            collinearity = base.read_coliearity(self.pairs_file)
            pairs = [[v[0], v[2]] for k in collinearity for v in k[1]]
        elif 'MATCH_SIZE' in p or '## Alignment' in p:
            collinearity = base.read_mcscanx(self.pairs_file)
            pairs = [[v[0], v[2]] for k in collinearity for v in k[1]]
        elif ',' in p:
            collinearity = pd.read_csv(self.pairs_file, header=None)
            pairs = collinearity.values.tolist()
        else:
            collinearity = pd.read_csv(self.pairs_file, header=None, sep='\t')
            pairs = collinearity.values.tolist()
        df = pd.DataFrame(pairs)
        df = df.drop_duplicates()
        df[0] = df[0].astype(str)
        df[1] = df[1].astype(str)
        df.index = df[0]+','+df[1]
        return df

    def run(self):
        path = os.getcwd()
        if not os.path.exists(self.pep_file):
            base.cds_to_pep(os.path.join(path, self.cds_file),
                            os.path.join(path, self.pep_file))
        cds = SeqIO.to_dict(SeqIO.parse(self.cds_file, "fasta"))
        pep = SeqIO.to_dict(SeqIO.parse(self.pep_file, "fasta"))
        df_pairs = self.auto_file()
        if os.path.exists(self.ks_file):
            ks = pd.read_csv(self.ks_file, sep='\t')
            ks = ks.drop_duplicates()
            ks['id'] = ks['id1']+','+ks['id2']
            df_pairs.drop(np.intersect1d(df_pairs.index,
                                         ks['id'].to_numpy()), inplace=True)
            ks_file = open(self.ks_file, 'a+')
        else:
            ks_file = open(self.ks_file, 'w')
            ks_file.write(
                '\t'.join(['id1', 'id2', 'ka_NG86', 'ks_NG86', 'ka_YN00', 'ks_YN00'])+'\n')
        df_pairs = df_pairs[(df_pairs[0].isin(cds.keys())) & (df_pairs[1].isin(
            cds.keys())) & (df_pairs[0].isin(pep.keys())) & (df_pairs[1].isin(pep.keys()))]
        pairs = df_pairs[[0, 1]].to_numpy()
        for k in pairs:
            SeqIO.write([cds[k[0]], cds[k[1]]], self.pair_cds_file, "fasta")
            SeqIO.write([pep[k[0]], pep[k[1]]], self.pair_pep_file, "fasta")
            kaks = self.pair_kaks(k)
            print(k, kaks)
            if kaks == None:
                continue
            ks_file.write('\t'.join([str(i) for i in list(k)+list(kaks)])+'\n')
        ks_file.close()
        for file in (self.pair_pep_file, self.pair_cds_file, self.mrtrans, self.pair_yn, self.prot_align_file, '2YN.dN', '2YN.dS', '2YN.t', 'rst', 'rst1', 'yn00.ctl', 'rub'):
            try:
                os.remove(file)
            except OSError:
                pass

    def pair_kaks(self, k):
        k[0], k[1] = k[0].replace('.', '_'), k[1].replace('.', '_')
        self.align()
        pal = self.pal2nal()
        if not pal:
            return []
        kaks = self.run_yn00()
        if kaks == None:
            return []
        kaks_new = [kaks[k[0]][k[1]]['NG86']['dN'], kaks[k[0]][k[1]]['NG86']
                    ['dS'], kaks[k[0]][k[1]]['YN00']['dN'], kaks[k[0]][k[1]]['YN00']['dS']]
        return kaks_new

    def align(self):
        if self.align_software == 'mafft':
            mafft_cline = MafftCommandline(
                cmd=self.mafft_path, input=self.pair_pep_file, auto=True)
            stdout, stderr = mafft_cline()
            align = AlignIO.read(StringIO(stdout), "fasta")
            AlignIO.write(align, self.prot_align_file, "fasta")
        if self.align_software == 'muscle':
            muscle_cline = MuscleCommandline(
                cmd=self.muscle_path, input=self.pair_pep_file, out=self.prot_align_file, seqtype="protein", clwstrict=True)
            stdout, stderr = muscle_cline()

    def pal2nal(self):
        args = ['perl', self.pal2nal_path, self.prot_align_file,
                self.pair_cds_file, '-output paml -nogap', '>'+self.mrtrans]
        command = ' '.join(args)
        try:
            os.system(command)
        except:
            return False
        return True

    def run_yn00(self):
        yn = yn00.Yn00()
        yn.alignment = self.mrtrans
        yn.out_file = self.pair_yn
        yn.set_options(icode=0, commonf3x4=0, weighting=0, verbose=1)
        try:
            run_result = yn.run(command=self.yn00_path)
        except:
            run_result = None
        return run_result



import re
import sys

import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import wgdi.base as base

class retain():
    def __init__(self, options):
        self.position = 'order'
        for k, v in options:
            setattr(self, str(k), v)
            print(str(k), ' = ', v)
        if hasattr(self, 'ylim'):
            self.ylim = [float(k) for k in self.ylim.split(',')]
        else:
            self.ylim = [0,1]
        self.colors = [str(k) for k in self.colors.split(',')]
        self.figsize = [float(k) for k in self.figsize.split(',')]

    def run(self):
        gff = base.newgff(self.gff)
        lens = base.newlens(self.lens, self.position)
        gff = gff[gff['chr'].isin(lens.index)]
        alignment = pd.read_csv(self.alignment,header=None, index_col=0)
        alignment = alignment.join(gff[['chr',self.position]], how='left')
        self.retain = self.align_chr(alignment)
        self.retain[self.retain.columns[:-2]
                    ].to_csv(self.savefile, sep='\t', header=None)
        fig, axs = plt.subplots(
            len(lens), 1, sharex=True, sharey=True, figsize=tuple(self.figsize))
        fig.add_subplot(111, frameon=False)
        align = dict(family='Arial', verticalalignment="center", horizontalalignment="center")
        plt.ylabel(self.ylabel+'\n\n\n\n', fontsize=20, **align)
        for spine in plt.gca().spines.values():
            spine.set_visible(False)
        plt.tick_params(top=False, bottom=False, left=False,
                        right=False, labelleft=False, labelbottom=False)
        groups = self.retain.groupby(['chr'])
        for i in range(len(lens)):
            group = groups.get_group(lens.index[i])
            for j in self.retain.columns[:-2]:
                axs[i].plot(group['order'].values, group[j].values,
                            linestyle='-', color=self.colors[j-1], linewidth=1)
            axs[i].spines['right'].set_visible(False)
            axs[i].spines['top'].set_visible(False)
            axs[i].set_ylim(self.ylim)
            axs[i].tick_params(labelsize=12)
        for i in range(len(lens)):
            x, y = axs[i].get_xlim()[1]*0.95, axs[i].get_ylim()[1]*0.5
            axs[i].text(x, y, self.refgenome+' ' +
                        str(lens.index[i]), fontsize=18, **align)
        plt.subplots_adjust(left=0.1, right=0.95, top=0.95, bottom=0.05)
        plt.savefig(self.savefig, dpi=500)
        plt.show()
        sys.exit(0)

    def align_chr(self, alignment):
        for i in alignment.columns[:-2]:
            alignment.loc[alignment[i].str.contains('\w', na=False), i] = 1
            alignment.loc[alignment[i] == '.', i] = 0
            alignment.loc[alignment[i] == ' ', i] = 0
            alignment[i].fillna(0, inplace=True)
            for chr, group in alignment.groupby(['chr']):
                a = self.retain(group[i].values.tolist())
                alignment.loc[group.index, i] = a
        return alignment

    def retain(self, arr):
        a = []
        for i in range(0, len(arr)):
            start, end = i-int(self.step), i+int(self.step)
            if start < 0:
                start = 0
            if end > len(arr):
                end = len(arr)
            ave = sum(arr[start:end])/(end-start)
            a.append(ave)
        return a



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

from setuptools import setup, find_packages

with open("README.md", "r",encoding='utf-8') as fh:
    long_description = fh.read()

with open('requirements.txt') as f:
    required = f.read().splitlines()

setup(
    name="WGDI",
    version="0.3.7",
    author="Pengchuan Sun",
    author_email="sunpengchuan@gmail.com",
    description="Whole Genome Duplication Identification",
    license="BSD License",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/SunPengChuan/wgdi",
    packages=find_packages(),
    package_data={'':['*.conf'],'wgdi': ['*.ini']},
    classifiers=[
        "Intended Audience :: Science/Research",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
    ],
    entry_points={
        'console_scripts':[
            'wgdi = wgdi.run:main',
        ]
      },
    zip_safe=True,
    install_requires=required
)